;===========================================================================================
; MouseHook.asm  GoAsm assembler
; MouseHook afficher les coordonnes souris dans bandeau infrieur
; Programm par AsmGges France
;===========================================================================================
; compile.bat:
; ..\..\bin\GoRC /r, res\moushook.rc
; ..\..\bin\GoAsm moushook.asm
; ..\..\bin\GoLink moushook.obj res\moushook.res user32.dll kernel32.dll gdi32.dll 
;                                                                     comctl32.dll
;===========================================================================================

; Inclusion des macros/quates/structures ncessaires au programme
; ----------------------------------------------------------------
  #include "..\..\inc\Windows.inc"
  #include "..\..\inc\all_api.inc"

  ID_FRANCE    = 17
  IDM_MOUSE    = 100
  ID_STATUSBAR = 1000

; Macro for wsprintf access
cinvoke(funcname,%1,%2,%3,%4,%5) = \ 
                               #if ARGCOUNT=1                     \ 
                                   invoke funcname                \ 
                               #elif ARGCOUNT=2                   \ 
                                   invoke funcname,%1             \ 
                               #elif ARGCOUNT=3                   \ 
                                   invoke funcname,%1,%2          \ 
                               #elif ARGCOUNT=4                   \ 
                                   invoke funcname,%1,%2,%3       \ 
                               #elif ARGCOUNT=5                   \ 
                                   invoke funcname,%1,%2,%3,%4    \ 
                               #elif ARGCOUNT=6                   \ 
                                   invoke funcname,%1,%2,%3,%4,%5 \ 
                               #endif                             \
                               #if ARGCOUNT>1                     \ 
                                   add esp,ARGCOUNT-1*4           \
                               #endif

;===========================================================================================
; Dbut du code programme
; -----------------------
.code

 start:
    ; Obtenir le handle du programme
    ; ------------------------------
    invoke GetModuleHandle,0
    mov [wc.hInstance],eax

    ; Chargement d'une icne spcifique
    ; ---------------------------------
    invoke LoadIcon,[wc.hInstance],ID_FRANCE
    mov [wc.hIcon],eax

    ; Chargement d'un curseur systme
    ; -------------------------------
    invoke LoadCursor,0,IDC_ARROW
    mov [wc.hCursor],eax

    ; Chargement d'un menu
    ; --------------------
    invoke LoadMenu,[wc.hInstance],IDM_MOUSE
    mov [hMenu],eax

    ; Obtenir la largeur en pixels de l'cran
    ; ---------------------------------------
    invoke GetSystemMetrics,SM_CXSCREEN
    sub eax,[WinW]
    shr eax,1
    mov [WinX],eax

    ; Valider la structure WNDCLASS
    ; -----------------------------
    invoke RegisterClass,addr wc

    ; Cration de la fenetre de l'application
    ; ---------------------------------------
    invoke CreateWindowEx,0,addr ClassName,addr Titre,WS_OVERLAPPEDWINDOW,\
                          [WinX],[WinY],[WinW],[WinH],0,[hMenu],[wc.hInstance],0
    mov [hWnd],eax

    ; Affiche et rafraichit la fenetre
    ; --------------------------------
    invoke ShowWindow,[hWnd],SW_SHOWDEFAULT
    invoke UpdateWindow,[hWnd]

 ; Boucle des messages
 ; -------------------
 MessageLoop:
    invoke GetMessage,addr msg,0,0,0
    or eax,eax
    jz >ExitProgram
    invoke TranslateMessage,addr msg
    invoke DispatchMessage,addr msg
    jmp <MessageLoop

 ExitProgram:
    ; Fermeture de l'application
    ; --------------------------
    invoke ExitProcess,[msg.wParam]

;===========================================================================================
; Procdure principale Windows
;-------------------------------------------------------------------------------------------
WindowProc frame @hWnd, @uMsg, @wParam, @lParam
    uses ebx,esi,edi

    cmp D[@uMsg],WM_CLOSE
    jne >.wmcreate
        invoke MessageBox,[@hWnd],addr MsgEnd,addr ClassName,MB_OK+MB_ICONINFORMATION
        invoke UnhookWindowsHookEx,[hMouseHook]
        invoke PostQuitMessage,0
        jmp >>.exit

    .wmcreate:
    cmp D[@uMsg],WM_CREATE
    jne >.wmsize
        ; Assure que la bibliothque de liaison dynamique de contrle commune est charge
        ; -------------------------------------------------------------------------------
        mov D[iccex.dwSize],sizeof INITCOMMONCONTROLSEX
        mov D[iccex.dwICC],ICC_DATE_CLASSES
        invoke InitCommonControlsEx,addr iccex
        ;invoke InitCommonControls

        invoke CreateStatusWindow,WS_CHILD+WS_VISIBLE,NULL,[@hWnd],ID_STATUSBAR
        mov [hWndStatus],eax

        invoke SendMessage,[hWndStatus],SB_SETPARTS,[nParts],addr statusWindowParts
        invoke SendMessage,[hWndStatus],SB_SETTEXT,1,addr ClassName
        jmp >>.exit

    .wmsize:
    cmp D[@uMsg],WM_SIZE
    jne >.defwindowproc
        invoke GetClientRect,[@hWnd],addr rect
        mov eax,[rect.right]

        sub eax,[rect.left]
        shr eax,1
        mov ebx,addr statusWindowParts
        mov [ebx],eax
        mov D[ebx+4],-1
        invoke SendMessage,[hWndStatus],SB_SETPARTS,[nParts],addr statusWindowParts

        invoke GetCurrentThreadId
        mov [nThreadID],eax
        invoke SetWindowsHookEx,WH_MOUSE,addr MouseHookProc,NULL,[nThreadID]

        invoke MoveWindow,[hWndStatus],0,0,0,0,TRUE
        jmp >.exit

    .defwindowproc:
        invoke DefWindowProc,[@hWnd],[@uMsg],[@wParam],[@lParam]
        ret

    .exit:
    mov eax,0
    ret
endf

;===========================================================================================
MouseHookProc frame @nCode, @wParam, @lParam

    cmp D[@nCode],0
    jbe >
        invoke CallNextHookEx,[hMouseHook],[@nCode],[@wParam],[@lParam]
        jmp >.exit
    :
        mov edx,[@lParam]              ; addr struct MOUSEHOOKSTRUCT
        mov eax,[edx]                  ; MOUSEHOOKSTRUCT.pt.x
        mov edx,[edx+4]                ; MOUSEHOOKSTRUCT.pt.y
        cinvoke(wsprintf,addr szBuffer,addr szFormat,eax,edx)
        invoke SendMessage,[hWndStatus],SB_SETTEXT,0,addr szBuffer

    .exit:
    mov eax,0
    ret
endf

;===========================================================================================
; Dfinitions des datas chaines / variables initialises ou non initialises / structures
;-------------------------------------------------------------------------------------------
.data

  Titre       db ' Mouse Hook',0
  ClassName   db ' AsmGges Win32',0
  MsgEnd      db ' AsmGges France @2006  ',0

  szFormat    db 'x: %u   y: %u',0
  align 4

  WinX        dd 0
  WinY        dd 100
  WinW        dd 300
  WinH        dd 244

  statusWindowParts dd 0,0
  nParts      dd 1

  szBuffer    db 40 dup ?
  align 4

  hWnd        dd ?
  hMouseHook  dd ?
  nLastError  dd ?
  nThreadID   dd ?
  hMenu       dd ?
  hWndStatus  dd ?

  wc          WNDCLASS              < CS_HREDRAW+CS_VREDRAW, addr WindowProc, 0, 0, 0,\
                                      0, 0, COLOR_WINDOW+1,0, addr ClassName >                                                                            
  msg         MSG                   <>
  iccex       INITCOMMONCONTROLSEX  <>
  rect        RECT                  <>

;===========================================================================================

